@@ -141,7 +141,7 @@ def format_item(x, timedelta_format=None, quote_strings=True):
         return format_timedelta(x, timedelta_format=timedelta_format)
     elif isinstance(x, (str, bytes)):
         return repr(x) if quote_strings else x
-    elif isinstance(x, (float, np.float_)):
+    elif np.issubdtype(type(x), np.floating):
         return f"{x:.4}"
     else:
         return str(x)
